#include "IMConfig.h"
#include  <time.h>

//Constructor
IMConfig::IMConfig() {
	memset(&ser, 0, sizeof(ser));
	ser.sin_family = AF_INET;
	//ser.sin_addr.s_addr = inet_addr(str2);
	//ser.sin_addr.s_addr = inet_addr("192.168.211.100");
	ser.sin_port = htons(0x4444);

	memset(&serviceIn, 0, sizeof(serviceIn));
	serviceIn.sin_family = AF_INET;
	serviceIn.sin_addr.s_addr = 0;
	serviceIn.sin_port = htons(0x5555);

	ListenSocketIn = socket(AF_INET, SOCK_DGRAM, 0);
	int option=1;
	setsockopt(ListenSocketIn, SOL_SOCKET, SO_BROADCAST, (char *)&option, sizeof (option));

	time_t tt = time(NULL);
	count = (int)tt;
}

int IMConfig::setSerIP(char* serIP) {
	ser.sin_addr.s_addr = inet_addr(serIP);
    return 0;
}

int IMConfig::bindSocket() {
	int binded=bind(ListenSocketIn, (struct sockaddr*)&serviceIn, sizeof(serviceIn));
	if (binded < 0) {
		fprintf(stderr, "Error binding: %d. Socket: %d.\n", binded, ListenSocketIn);
		return binded;
	}
	return 0;
}

int IMConfig::sendUDP (char* udp)
{
	char send[2000];

	int len = (int)strlen(udp) + 1;
	int i;

	*(int*)&(send[0]) = 0;
	*(int*)&(send[2]) = GL_CODE_RX_COMMAND;
	*(int*)&(send[6]) = count ++;

	memcpy(send + 10, udp, len);
	len += 10;

    int er = 0;
	for (i = 0; i < 3; i ++) {
		er = sendto(ListenSocketIn, (char*)send, len, 0, (struct sockaddr*)&ser, sizeof(ser));
	}
//	printf(":Send %s, ListenSocketIn: %d, len: %d, dstip: %s, count: %d\n", send+10, ListenSocketIn, len, inet_ntoa(ser.sin_addr), count);
    return er;
}

unsigned int IMConfig::receiveUDP () {
	memset(&received.data, 0, sizeof(received.data));

	char udp[2000];
	struct sockaddr_in ser2;
	int serlen=sizeof(struct sockaddr_in);	
	ser2.sin_addr.s_addr = 1;
	int bytesRecv = recvfrom(ListenSocketIn, (char*)udp, sizeof(udp), 0, (struct sockaddr*)&ser2, (socklen_t*)&serlen);

	processPacketUDP(udp, bytesRecv);
	return ser2.sin_addr.s_addr;
}

void IMConfig::processPacketUDP (char* data, int len) {
	//if (*(unsigned short*) received.data != 0)
	//	return;

	data += sizeof(unsigned short);
	len -= 2;

	received.type = *(unsigned int*)data;
	data += sizeof(unsigned int);
	len -= sizeof(unsigned int);
	
	received.counter = *(unsigned int*)data;
	data += sizeof(unsigned int);
	len -= sizeof(unsigned int);

	strcpy(received.data,data);
}

int IMConfig::sendRegister() {
	char udp[200];
	memset(&udp, 0, sizeof(udp));
	strcpy(udp, "REGISTER");
	return sendUDP(udp);
}

int IMConfig::sendInfo() {
	char udp[200];
	memset(&udp, 0, sizeof(udp));
	strcpy(udp, "INFO");
	return sendUDP(udp);
}

int IMConfig::sendReset() {
	char udp[200];
	memset(&udp, 0, sizeof(udp));
	strcpy(udp, "RESET");
	return sendUDP(udp);
}

int IMConfig::sendSetIP(char* IP) {
	char udp[200];
	memset(&udp, 0, sizeof(udp));
	strcpy(udp, "SETIP ");
	strcpy(udp+6, IP);
	return sendUDP(udp);
}

int IMConfig::sendLicense(char* License) {
	char udp[200];
	memset(&udp, 0, sizeof(udp));
	strcpy(udp, "SETLICENSE ");
	strcpy(udp+11, License);
	return sendUDP(udp);
}

int IMConfig::sendFile(char* fileName) {
	sendRegister();
	FILE* f = fopen(fileName, "rb");
	if (!f) {
		fprintf(stderr, "Error openning file %s . Error: %s\n", fileName, strerror(errno));
		return 0;
	}
	
	unsigned char buf[128*1024];
	memset(buf, 0, sizeof(buf));
	int len = (int)fread(buf, 1, sizeof(buf), f);
	fclose(f);
	len = (len + 255) / 256 * 256;
	unsigned int cs = calcconsum(buf, len);
	
	printf("Loading will take about 20 seconds.\n");
	
	char send[200];
	int step;
	for (step = 0; step < len / 32; step ++) {
		sprintf(send, "UPWARE %d ", step*32);
		for (int i = 0; i < 32; i ++)
			sprintf(send + strlen(send), "%02X", buf[step*32 + i]);
		sendUDP(send);
		usleep(16);

		unsigned int prev_counter=received.counter;
		while (strncmp(received.data, "ANSWER UPWARE", 13) != 0 || received.counter == prev_counter) {
			receiveUDP();
		}

		if (received.type == GL_CODE_TX_DATA) {
			printf("Received Type: GL_CODE_TX_DATA bytesRecv: %d, Counter: %lu, Received: %s\n", received.len, received.counter, received.data);
		} else if (received.type == GL_CODE_TX_COMMAND) {
			printf("Received Type: GL_CODE_TX_COMMAND bytesRecv: %d, Counter: %lu, Received: %s\n", received.len, received.counter, received.data);
		} else {
			printf("Received Type: %lu bytesRecv: %d, Counter: %lu, Received: %s\n", received.type, received.len, received.counter, received.data);
		}
	}
	
	sprintf(send, "UPWARE %d %02X%02X%02X%02X", step*32, (unsigned char)(cs >> 0), (unsigned char)(cs >> 8), (unsigned char)(cs >> 16), (unsigned char)(cs >> 24));
	sendUDP(send);
	printf("Loading completed. Try to send INFO after about 10 seconds.\n");
	sleep(10);
	
	return 0;
}

int IMConfig::sendDirect(char* str)
{
	return sendUDP(str);
}


unsigned int IMConfig::calcconsum (unsigned char *data, int len  ) {
	unsigned char b1=0, b2=0, b3=0, b4=0;
	int i;

	for (i = 0; i < len; i ++) {
		b1 += data[i];
		b2 ^= data[i];
		b3 += (unsigned char)(data[i] * data[i] / 256);
		b4 += (unsigned char)(i * data[i]);
	}
	return b1 + (b2 << 8) + (b3 << 16) + (b4 << 24);
}
